package com.persagy.server.controllers.dict

import com.persagy.server.datacenter.models.entities.dict.DictBatchProcessing
import com.persagy.server.datacenter.models.entities.dictnew.DefClass
import com.persagy.server.datacenter.models.entities.dictnew.DefFuncId
import com.persagy.server.datacenter.models.entities.dictnew.DefFuncIdProject
import com.persagy.server.datacenter.models.requests.CategoryRequest
import com.persagy.server.services.dict.dictnew.DictDefClassService
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.annotations.Api
import io.swagger.annotations.ApiOperation
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

/**
 * 字典
 *
 * @author wx  <zhangweixin@sagacloud.cn>
 * @date  2020/9/23 15:30
 */
@Api(tags = ["040、字典"])
@RestController
@RequestMapping("/dict")
class DictControllers {

    /**
     * 字典查询所有设备类型、部件所有类型、系统类型、专业类型
     *
     * @param  categoryRequest        查询信息条件
     * @return 查询结果
     */
    @ApiOperation(value = "类型-字典查询所有设备类型、部件所有类型、系统类型、专业类型", notes = "输入设备类型：equipment，输入部件类型：component,输入系统类型：system,输入专业类型：major," +
            "专业和系统：majorSystem，业务空间类型 space ,专业下的设备类型 majorEquipment ,设备和部件equipmentAndComponent,设备类型要以及对应的部件equipmentComponent，专业下的系统，系统下的设备majorSystemEquip ,资产 property ")
    @PostMapping(value = ["/category"])
    fun categoryQuery(@RequestBody categoryRequest: CategoryRequest): SQueryResponse<DefClass> {
        return DictDefClassService.equipQuery(categoryRequest)
    } // Fun categoryQuery()

    /**
     * 项目字典
     *
     * @param categoryRequest   查询条件
     * @return 查询结构
     */
    @ApiOperation(value ="查询项目下类型的信息点",notes = "")
    @PostMapping(value = ["/query"])
    fun categoryInfoQuery(@RequestBody categoryRequest: CategoryRequest): SQueryResponse<DefFuncId>{
        return DictDefClassService.categoryInfoQuery(categoryRequest)
    } // Fun query()

    /**
     * 项目定制化设置
     *
     * @param request   信息点对象列表
     * @return 返回结果
     */
    @ApiOperation(value = "设置-项目定制化字典", notes = "")
    @PostMapping(value = ["/setting-dic-project"])
    fun settingDict(@RequestBody request: SCreateRequest<DefFuncIdProject>): SCreateResponse<DefFuncIdProject> {
        return DictDefClassService.settingDict(request)
    } // Fun settingDict()

    /**
     * 重置-根据类型项目id 具体对象类型重置
     *
     * @param categoryRequest 类型
     * @return 返回结果
     */
    @ApiOperation(value = "重置-根据类型项目id 具体对象类型重置", notes = "重置项目下具体的类别")
    @PostMapping(value = ["/reset-type"])
    fun resetType(@RequestBody categoryRequest: CategoryRequest): SBaseResponse {
        return DictDefClassService.resetType(categoryRequest)
    } // Fun resetType()

    /**
     * 重置-根据项目id
     */
    @ApiOperation(value = "重置-根据项目id", notes = "重置项目下所有得定制信息点")
    @PostMapping(value = ["/reset-dic"])
    fun resetProjectId(): SBaseResponse{
        return DictDefClassService.resetProjectId()
    } // Fun reset()

    /**
     * 批量处理字典信息操作
     *
     * @param request   条件
     * @return 返回结果
     */
    @ApiOperation(value = "批量处理字典信息操作", notes = "方案id ：全新设定（schemeZero）、方案1（schemeOne）、方案2（schemeTwo），" +
            "注意SchemeId为必填字段 ，优先级在列表中[R,S,M] ,一级标签[技术参数，基本参数] 列表为String类型")
    @PostMapping(value = ["/dict-batch"])
    fun createList(@RequestBody request: DictBatchProcessing): SCreateResponse<DictBatchProcessing>{
        return DictDefClassService.setSchemeProject(request)
    } // Function createList()

} // Class DictControllers