package com.persagy.server.service

import com.persagy.base.extensions.toJson
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.rel.REqSyList
import com.persagy.server.datacenter.models.entities.rel.RSyEq
import com.persagy.server.datacenter.models.entities.rel.RSyEqList
import com.persagy.server.service.base.RService
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse


/**
 * 系统下的设备设施关系服务
 *
 * @author 张维新
 */
object RSyEqService : RService<RSyEq>(SMybatisDao(RSyEq::class.java)) {

    /**
     * 创建系统下设备关系
     *
     * @param   rSyEq   系统和设备的关系对象
     */
    fun create(rSyEq: RSyEq): SCreateResponse<RSyEq> {
        return try {
//            rSyEq.id = IdUtils.uuidCreate()
            val isInsert = insert(rSyEq)
            if (isInsert){
                SCreateResponse(SResponseType.success)
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function create()

    /**
     * 根据对象删除
     *
     * @param   rSyEq      关系对象实例
     * @return
     */
     fun deletes(rSyEq: RSyEq): SBaseResponse {
        return try {
                val isDelete = delete(rSyEq)
                /** 删除成功 */
                if (isDelete) {
                    val sBaseResponse = SBaseResponse(SResponseType.success)
                    sBaseResponse.message = rSyEq.toJson()
                    sBaseResponse
                } else {
                    SBaseResponse(SResponseType.failure)
                }
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure, e.message!!)
        }
    } // Function deletes()

    /**
     * 创建系统下设备关系
     *
     * @param   rSyEqList   系统和设备的关系对象列表
     */
    fun createSy(rSyEqList: RSyEqList): SCreateResponse<RSyEq> {
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            delete("sysID = '${rSyEqList.sysId}'")
            for (eqId in rSyEqList.listEquipId!!){
                val rSyEq = RSyEq()
//                rSyEq.id = IdUtils.uuidCreate()
                rSyEq.equipId = eqId
                rSyEq.sysId =  rSyEqList.sysId
                val replace = replace(rSyEq)
                if (replace){
                    successList.add(rSyEq)
                }else{
                    failureList.add(rSyEq)
                }
            }
            if (successList.size>0){
                val sCreateResponse = SCreateResponse<RSyEq>(SResponseType.success)
                if (failureList.size>0){
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                }
                sCreateResponse
            }else{
               SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createSy()

    /**
     * 创建系统下设备关系
     *
     * @param   rSyEqList   系统和设备的关系对象列表
     */
    fun createsEq(rSyEqList: REqSyList): SCreateResponse<RSyEq> {
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            delete("equipId = '${rSyEqList.equipId}'")
            for (eqId in rSyEqList.sysIdList!!){
                val rSyEq = RSyEq()
//                rSyEq.id = IdUtils.uuidCreate()
                rSyEq.equipId = rSyEqList.equipId
                rSyEq.sysId = eqId
                val replace = replace(rSyEq)
                if (replace){
                    successList.add(rSyEq)
                }else{
                    failureList.add(rSyEq)
                }
            }
            if (successList.size>0){
                if (failureList.size>0){
                    val sCreateResponse = SCreateResponse<RSyEq>(SResponseType.success)
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                    sCreateResponse
                }else{
                    val sCreateResponse = SCreateResponse<RSyEq>(SResponseType.success)
                    sCreateResponse
                }
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createsEq()

} // Object RSyEqService